--[[
        Engine Of Love - a framework for the LÖVE game engine
        Copyright (C) 2013 vitaminx (vitaminx@callistix.net)

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see <http://www.gnu.org/licenses/>.
--]]

eol.debug={}

function eol.debug.init()
    -- get default values from prefs
    eol.debug.visible=prefs.debug.visible.v
    eol.debug.page=prefs.debug.page.v

    -- load font
    if prefs.debug.font_type.v=="pixmap" then eol.debug.font=love.graphics.newImageFont(prefs.debug.font_path.v,eol.charmap)
    elseif prefs.debug.font_type.v=="ttf" then eol.debug.font=love.graphics.newFont(prefs.debug.font_path.v,prefs.debug.font_size.v)
    else eol.debug.font=love.graphics.newFont(prefs.debug.font_size.v) end

    -- calculate horizontal gap inside debug rectangle
    eol.debug.hgap=(eol.debug.font:getWidth("1")/2)+prefs.debug.gap_off.v

    -- calculate colors for background and font
    eol.debug.color_r,eol.debug.color_g,eol.debug.color_b=eol.misc.convertrgb(prefs.debug.color.v)
    eol.debug.bg_color_r,eol.debug.bg_color_g,eol.debug.bg_color_b=eol.misc.convertrgb(prefs.debug.bg_color.v)

    -- populate system debug table
    eol.debug.system = {
        { n="memory usage",         f=function() return gcinfo().." kB" end  },
        { n="fps limit",            f=function() return prefs.screen.fpslimit.v end },
        { n="fps",                  f=love.timer.getFPS },
        { n="screen resolution",    f=function() return love.graphics.getWidth().."x"..love.graphics.getHeight() end },
        { n="canvas resolution",    f=function() return prefs.screen.canvas_w.v.."x"..prefs.screen.canvas_h.v end },
        { n="canvas scaling",       f=function() return "x"..eol.screen.scale-eol.screen.scale%0.01 end },
        { n="canvas multiscale",    f=function() return prefs.screen.multiscale.v end  },
        { n="canvas support",       f=function() return eol.screen.canvas_sup end  },
        { n="fullscreen",           f=function() local _w,_h,_fs=love.graphics.getMode() return _fs end },
        { n="vsync",                f=function() local _w,_h,_fs,_vs=love.graphics.getMode() return _vs end },
        { n="fsaa",                 f=function() local _w,_h,_fs,_vs,_aa=love.graphics.getMode() return _aa end },
        { n="audio enabled",        f=function() return prefs.audio.enabled.v end },
        { n="audio muted",          f=function() return eol.audio.muted end },
        { n="audio volume",         f=function() return eol.audio.volume.."%" end },
    }
end

function eol.debug.update(_dt)
    if prefs.debug.enabled.v and eol.debug.visible then
        local _d

        -- choose between user and system debug info
        if eol.debug.page=="user" and eol.debug.user then _d=eol.debug.user
        else _d=eol.debug.system end

        if _d then
            -- execute functions from debug table
            for _i,_v in ipairs(_d) do _d[_i].v=_v.f() end

            -- format and align name column
            local _c=0
            for _i,_v in ipairs(_d) do if #_v.n>_c then _c=#_v.n end end
            for _i,_v in ipairs(_d) do _d[_i].fn=string.format("%-".._c.."s",_v.n).." = " end

            -- combine keys and values to a formated string
            eol.debug.string=""
            eol.debug.width=0
            for _i=1,#_d do
                local _v=tostring(_d[_i].v)
                local _w=eol.debug.font:getWidth(_d[_i].fn.._v)
                if _w>eol.debug.width then eol.debug.width=_w end
                eol.debug.string=eol.debug.string.._d[_i].fn.._v.."\n"
            end
        end

        -- calculate dimensions of debug rectangle
        eol.debug.width=0
        for _l in string.gmatch(eol.debug.string,"[^\r\n]+") do
            if eol.debug.font:getWidth(_l)>eol.debug.width then eol.debug.width=eol.debug.font:getWidth(_l) end
        end
        eol.debug.height=eol.debug.font:getHeight(eol.debug.string)*#_d

        -- calculate y position of debug rectangle
        if prefs.debug.valign.v=="bottom" then eol.debug.ypos=prefs.screen.canvas_h.v-eol.debug.height-prefs.debug.voff.v-prefs.debug.height_off.v
        elseif prefs.debug.valign.v=="center" then eol.debug.ypos=((prefs.screen.canvas_h.v/2)-(eol.debug.height/2)+prefs.debug.voff.v)-(prefs.debug.height_off.v/2)
        else eol.debug.ypos=prefs.debug.voff.v end
    
        -- calculate x position of debug rectangle
        if prefs.debug.halign.v=="right" then eol.debug.xpos=prefs.screen.canvas_w.v-eol.debug.width-prefs.debug.hoff.v-eol.debug.hgap*2
        elseif prefs.debug.halign.v=="center" then eol.debug.xpos=(prefs.screen.canvas_w.v/2)-(eol.debug.width/2)+(prefs.debug.hoff.v-eol.debug.hgap)
        else eol.debug.xpos=prefs.debug.hoff.v end
    end
end

function eol.debug.draw()
    if prefs.debug.enabled.v and eol.debug.visible and eol.debug.string then
        -- draw background rectangle
        love.graphics.setColor(eol.debug.bg_color_r,eol.debug.bg_color_g,eol.debug.bg_color_b,(1-prefs.debug.bg_trans.v)*255)
        love.graphics.rectangle("fill",eol.screen.ofs_x_nocan+eol.debug.xpos,eol.screen.ofs_y_nocan+eol.debug.ypos,eol.debug.width+eol.debug.hgap*2,eol.debug.height+prefs.debug.height_off.v)

        -- draw debug string
        love.graphics.setFont(eol.debug.font)
        love.graphics.setColor(eol.debug.color_r,eol.debug.color_g,eol.debug.color_b,(1-prefs.debug.trans.v)*255)
        love.graphics.print(eol.debug.string,eol.screen.ofs_x_nocan+eol.debug.xpos+eol.debug.hgap,eol.screen.ofs_y_nocan+eol.debug.ypos+prefs.debug.vtext_off.v)
    end
end

function eol.debug.keydown(_key)
    if prefs.debug.enabled.v then
        for _k,_v in pairs(keys.debug) do
            -- toggle visibility of debug info if toggle key pressed
            if _k=="toggle" and _v.v==_key then
                eol.debug.visible=not eol.debug.visible
                break end
            -- switch between system and user debug page if page key pressed
            if _k=="page" and _v.v==_key then
                if eol.debug.page=="user" then eol.debug.page="system"
                else
                    if eol.debug.user then eol.debug.page="user" end
                break end
            end
        end
    end
end
